@@ -1,5 +1,5 @@
 /*****************************************************************************\
- * Copyright 2005, 2006 Niels Lohmann, Christian Gierds                      *
+ * Copyright 2005, 2006, 2007 Niels Lohmann, Christian Gierds                *
  *                                                                           *
  * This file is part of GNU BPEL2oWFN.                                       *
  *                                                                           *
@@ -25,18 +25,17 @@
  *
  * \author  Niels Lohmann <nlohmann@informatik.hu-berlin.de>,
  *          Christian Gierds <gierds@informatik.hu-berlin.de>,
- *          last changes of: \$Author: nielslohmann $
+ *          last changes of: \$Author: znamirow $
  * 
  * \since   2005/07/02
  *
- * \date    \$Date: 2007/04/19 12:07:46 $
+ * \date    \$Date: 2007/05/11 12:53:46 $
  * 
  * \note    This file is part of the tool BPEL2oWFN and was created during the
  *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
- *          http://www.informatik.hu-berlin.de/top/forschung/projekte/tools4bpel
- *          for details.
+ *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.90 $
+ * \version \$Revision: 1.115 $
  */
 
 
@@ -56,6 +55,8 @@
 #include "helpers.h"
 #include "frontend-parser.h"
 #include "globals.h"
+#include "extension-wsdl.h"
+#include "extension-chor.h"
 
 using std::cerr;
 using std::cout;
@@ -75,13 +76,12 @@ using std::endl;
  * \param myid an id of an AST node
  * \param mytype value of the type of the node using the token values defined
  *               by flex and bison
- *
- * \todo "real" initialization
  */
 ASTE::ASTE(unsigned int myid, unsigned int mytype) :
-  id(myid), type(mytype), controlFlow(POSITIVECF), 
-  plRoleDetails(NULL), isStartActivity(false), cyclic(false),
-  sourceActivity(0), targetActivity(0), max_occurrences(1), max_loops(UINT_MAX), enclosedFH(0), enclosedCH(0), drawn(false)
+  id(myid), type(mytype), controlFlow(POSITIVECF), visConnection("none"), secVisConnection("none"), 
+  plRoleDetails(NULL), isStartActivity(false), cyclic(false), highlighted(false), isUserDefined(true), callable(true),
+  sourceActivity(0), targetActivity(0), max_occurrences(1), max_loops(UINT_MAX), enclosedFH(0), enclosedCH(0), channel_instances(0),
+  channel_unique_instances(0), drawn(false), partnerLinkType(NULL)
 {
   assert(myid != 0);
 
@@ -89,38 +89,75 @@ ASTE::ASTE(unsigned int myid, unsigned int mytype) :
 
   if (globals::parsing)
   {
+    // we are parsing: these activities are explicit
     switch (mytype)
     {
       case(K_LINK):	globals::process_information.links++; break;
       case(K_SCOPE):	globals::process_information.scopes++; break;
       case(K_VARIABLE):	globals::process_information.variables++; break;
 
-      case(K_EMPTY): case(K_INVOKE): case(K_RECEIVE): case(K_REPLY): case(K_ASSIGN):
-      case(K_VALIDATE): case(K_WAIT): case(K_THROW): case(K_RETHROW):
-      case(K_EXIT): case(K_COMPENSATE): case(K_COMPENSATESCOPE):
+      // basic activity
+      case(K_EMPTY):
+      case(K_INVOKE):
+      case(K_RECEIVE):
+      case(K_REPLY):
+      case(K_ASSIGN):
+      case(K_VALIDATE):
+      case(K_WAIT):
+      case(K_THROW):
+      case(K_RETHROW):
+      case(K_EXIT):
+      case(K_COMPENSATE):
+      case(K_COMPENSATESCOPE):
+      case(K_OPAQUEACTIVITY):
 			globals::process_information.basic_activities++; break;
 
-      case(K_WHILE): case(K_REPEATUNTIL): case(K_SEQUENCE): case(K_FLOW):
-      case(K_PICK): case(K_IF): case(K_FOREACH):
+      // structured activities
+      case(K_WHILE):
+      case(K_REPEATUNTIL):
+      case(K_SEQUENCE):
+      case(K_FLOW):
+      case(K_PICK):
+      case(K_IF):
+      case(K_FOREACH):
 			globals::process_information.structured_activities++; break;
     }
   }
   else
   {
+    // we are not parsing any more: these activities are implicit
     switch (mytype)
     {
       case(K_SCOPE):
-      case(K_EMPTY): case(K_INVOKE): case(K_RECEIVE): case(K_REPLY): case(K_ASSIGN):
-      case(K_VALIDATE): case(K_WAIT): case(K_THROW): case(K_RETHROW):
-      case(K_EXIT): case(K_COMPENSATE): case(K_COMPENSATESCOPE):
-      case(K_WHILE): case(K_REPEATUNTIL): case(K_SEQUENCE): case(K_FLOW):
-      case(K_PICK): case(K_IF): case(K_FOREACH):
-	globals::process_information.implicit_activities++; break;
+      case(K_EMPTY):
+      case(K_INVOKE):
+      case(K_RECEIVE):
+      case(K_REPLY):
+      case(K_ASSIGN):
+      case(K_VALIDATE):
+      case(K_WAIT):
+      case(K_THROW):
+      case(K_RETHROW):
+      case(K_EXIT):
+      case(K_COMPENSATE):
+      case(K_COMPENSATESCOPE):
+      case(K_OPAQUEACTIVITY):
+      case(K_WHILE):
+      case(K_REPEATUNTIL):
+      case(K_SEQUENCE):
+      case(K_FLOW):
+      case(K_PICK):
+      case(K_IF):
+      case(K_FOREACH):
+	                globals::process_information.implicit_activities++; break;
     }
   }
 }
 
 
+
+
+
 /*!
  * \brief destructor
  */
@@ -193,6 +230,15 @@ void ASTE::checkAttributes()
 	string values[] = {"no", "no", "no", "no"};
 	setStandardAttributes(names, values, 3);
 
+	// read the optional BPEL4Chor file to find the number of instances of this process
+    	if (globals::choreography_filename != "")
+	{
+	  globals::instances_of_current_process = globals::BPEL4ChorInfo.instances(attributes["targetNamespace"]);
+	  if (globals::instances_of_current_process == 0)
+	    genericError(137, "targetNamespace `" + attributes["targetNamespace"] + "' not found in BPEL4Chor topology file", attributes["referenceLine"], ERRORLEVEL_ERROR);
+	}
+
+	// if the process is abstract, print a notice
 	if (attributes["abstractProcess"] == "yes" || attributes["abstractProcessProfile"] != "")
 	{
 	  globals::abstract_process = true;
@@ -293,7 +339,7 @@ void ASTE::checkAttributes()
 	assert(attributes["exitOnStandardFault"] != "");
 
 	/* no break here */
-      }
+      }      
 
     case(K_EMPTY):
     case(K_INVOKE):
@@ -366,6 +412,7 @@ void ASTE::checkAttributes()
 
 	string catchString = toString(parentScopeId)
 	  + "|" + attributes["faultName"]
+	  + "|" + attributes["faultVariable"]
 	  + "|" + attributes["faultElement"]
 	  + "|" + attributes["faultMessageType"];
 
@@ -430,18 +477,28 @@ void ASTE::checkAttributes()
 
     case(K_INVOKE):
       {
-      	string required[] = {"partnerLink", "operation"};
-        checkRequiredAttributes(required, 2);
+	if (globals::choreography_filename == "")
+	{
+	  string required[] = {"partnerLink", "operation"};
+	  checkRequiredAttributes(required, 2);
+	}
+
 	checkAttributeType("inputVariable", T_BPELVARIABLENAME);
 	checkAttributeType("outputVariable", T_BPELVARIABLENAME);
+
 	break;
       }
 
     case(K_ONMESSAGE):
       {
-      	string required[] = {"partnerLink", "operation"};
-        checkRequiredAttributes(required, 2);
+	if (globals::choreography_filename == "")
+	{
+	  string required[] = {"partnerLink", "operation"};
+	  checkRequiredAttributes(required, 2);
+	}
+
 	checkAttributeType("variable", T_BPELVARIABLENAME);
+
 	break;
       }
     
@@ -485,8 +542,12 @@ void ASTE::checkAttributes()
 
     case(K_RECEIVE):
       {
-      	string required[] = {"partnerLink", "operation"};
-        checkRequiredAttributes(required, 2);
+	if (globals::choreography_filename == "")
+	{
+	  string required[] = {"partnerLink", "operation"};
+	  checkRequiredAttributes(required, 2);
+	}
+
 	checkAttributeType("createInstance", T_BOOLEAN);
 	checkAttributeType("variable", T_BPELVARIABLENAME);
 	
@@ -504,9 +565,14 @@ void ASTE::checkAttributes()
 
     case(K_REPLY):
       {
-      	string required[] = {"partnerLink", "operation"};
-        checkRequiredAttributes(required, 2);
+	if (globals::choreography_filename == "")
+	{
+	  string required[] = {"partnerLink", "operation"};
+	  checkRequiredAttributes(required, 2);
+	}
+
 	checkAttributeType("variable", T_BPELVARIABLENAME);
+
 	break;
       }
 
@@ -689,10 +755,66 @@ void ASTE::checkAttributeType(string attribute, attributeType type)
  *      for input and output channels. Thus, the Petri net unparser cannot find
  *      the channel place.
  *
+ * \todo Synchronous invoke in BPEL4Chor?
+ *
  * \return name of the channel
  */
 string ASTE::createChannel(bool synchronousCommunication)
 {
+  // in case of BPEL4Chor, we can read the channel name from the choreography
+  if (globals::choreography_filename != "")
+  {
+    string channelName = globals::BPEL4ChorInfo.channel_name(id);
+
+    switch (type)
+    {
+      // receiving activity
+      case(K_RECEIVE):
+      case(K_ONMESSAGE):
+	{
+	  // depending on the channel count, create one or more input channel(s)
+	  pair<unsigned int, bool> result = globals::BPEL4ChorInfo.channel_count(id, false);
+	  if (result.first != 0 && result.first != UINT_MAX)
+	  {
+	    globals::ASTE_inputChannels[channelName] = result.first;
+
+	    if (result.second)
+	      channel_unique_instances = result.first; // a unique instance
+	    else
+	      channel_instances = 1; // an iterator
+	  }
+	  else
+	    globals::ASTE_inputChannels[channelName] = 1;
+
+	  break;
+	}
+
+      // sending activity
+      case(K_INVOKE):
+      case(K_REPLY):
+	{
+	  // depending on the channel count, create one or more output channel(s)
+	  pair<unsigned int, bool> result = globals::BPEL4ChorInfo.channel_count(id, true);
+	  if (result.first != 0 && result.first != UINT_MAX)
+	  {
+	    globals::ASTE_outputChannels[channelName] = result.first;
+
+	    if (result.second)
+	      channel_unique_instances = result.first; // a unique instance
+	    else
+	      channel_instances = 1; // an iterator
+	  }
+	  else
+	    globals::ASTE_outputChannels[channelName] = 1;
+
+	  break;
+	}
+    }
+
+    return channelName;
+  }
+
+
   string channelName = attributes["partnerLink"] + "." + attributes["operation"];
   if (channelName == ".")
   {
@@ -709,7 +831,8 @@ string ASTE::createChannel(bool synchronousCommunication)
         {
           channelName = plRoleDetails->partnerRole + "." + plRoleDetails->myRole + "." + attributes["operation"];
         }
-	globals::ASTE_inputChannels.insert(channelName);
+//	globals::ASTE_inputChannels.insert(channelName);
+	globals::ASTE_inputChannels[channelName] = 1;
 	break;
       }
 
@@ -720,7 +843,8 @@ string ASTE::createChannel(bool synchronousCommunication)
         {
           channelName = plRoleDetails->myRole + "." + plRoleDetails->partnerRole + "." + attributes["operation"];
         }
-	globals::ASTE_outputChannels.insert(channelName);
+//	globals::ASTE_outputChannels.insert(channelName);
+	globals::ASTE_outputChannels[channelName] = 1;
 
 	if (synchronousCommunication)
         {
@@ -730,7 +854,8 @@ string ASTE::createChannel(bool synchronousCommunication)
             channelName2 = plRoleDetails->partnerRole + "." + plRoleDetails->myRole + "." + attributes["operation"];
           }
           this->channelName2 = channelName2;
-	  globals::ASTE_inputChannels.insert(channelName2);
+	  globals::ASTE_inputChannels[channelName2] = 1;
+//	  globals::ASTE_inputChannels.insert(channelName2);
         }
 
 	break;
@@ -792,7 +917,6 @@ bool ASTE::findIsolatedAncestor()
 
 
 
-
 /******************************************************************************
  * functions to define entities like variables, links, etc.
  *****************************************************************************/
@@ -879,6 +1003,20 @@ string ASTE::definePartnerLink()
   {
     globals::ASTE_partnerLinkNames.insert(name);
     globals::ASTE_partnerLinks[ attributes["name"] ] = id;
+
+
+    // find partnerLinkType_name
+    partnerLinkType = globals::WSDLInfo.partnerLinkTypes[attributes["partnerLinkType"]];
+
+    if (globals::wsdl_filename != "")
+    {
+      if (partnerLinkType == NULL)
+	genericError(130, "partnerLinkType `" + attributes["partnerLinkType"] + "' was not defined in WSDL file", attributes["referenceLine"], ERRORLEVER_WARNING);    
+      else
+	if (!globals::WSDLInfo.checkPartnerLinkType(partnerLinkType, attributes["partnerRole"]))
+	  genericError(129, "role `" + attributes["partnerRole"] + "' was not defined for partnerLinkType `" + attributes["partnerLinkType"] + "' in WSDL file", attributes["referenceLine"], ERRORLEVER_WARNING);
+    }
+    
   }
 
   return name;
@@ -888,6 +1026,80 @@ string ASTE::definePartnerLink()
 
 
 
+/*!
+ * \brief enters a possibly triggered fault to the appropriate set
+ *
+ * \todo comment me
+ */
+void ASTE::enterFault(string fault)
+{
+  if ( fault == "" )
+  {
+    return;
+  }
+
+  if (fault.find_first_of(":") < fault.npos)
+  {
+    fault = fault.substr( fault.find_first_of(":") + 1 );
+  }
+  possiblyTriggeredFaults.insert( fault );
+  if ( id > 1 )
+  {
+    globals::ASTEmap[parentScopeId]->enterFault( fault );
+  }
+}
+
+/*!
+ * \brief enters a possibly triggered fault to the appropriate set
+ *
+ * \todo comment me
+ */
+void ASTE::enterFault(WSDL_PartnerLinkType * plt)
+{
+  if ( plt == NULL )
+  {
+    return;
+  }
+  if ( plt->role1.second != NULL )
+  {
+    for (map<string, WSDL_Operation*>::iterator it = plt->role1.second->Operations.begin(); it != plt->role1.second->Operations.begin(); it++)
+      if ( it->second != NULL )
+        enterFault( it->second->faultName );
+  }
+  if ( plt->role2.second != NULL )
+  {
+    for (map<string, WSDL_Operation*>::iterator it2 = plt->role2.second->Operations.begin(); it2 != plt->role2.second->Operations.begin(); it2++)
+      if ( it2->second != NULL )
+        enterFault( it2->second->faultName );
+  }
+  
+}
+
+/*!
+ * \brief removes a possibly triggered fault from the appropriate set because it was caught
+ *
+ * \todo comment me
+ */
+void ASTE::removeFault(string fault)
+{
+  if ( fault == "" )
+  {
+    return;
+  }
+
+  if (fault.find_first_of(":") < fault.npos)
+  {
+    fault = fault.substr( fault.find_first_of(":") + 1 );
+  }
+  possiblyTriggeredFaults.erase( fault );
+  if ( id > 1 )
+  {
+    globals::ASTEmap[parentScopeId]->removeFault( fault );
+  }
+}
+
+
+
 /******************************************************************************
  * functions to check entities like variables, links, etc.
  *****************************************************************************/
@@ -976,6 +1188,7 @@ string ASTE::checkLink()
 string ASTE::checkPartnerLink()
 {
   string partnerLinkName = attributes["partnerLink"];
+
   if (partnerLinkName == "")
     return partnerLinkName;
 
@@ -986,14 +1199,21 @@ string ASTE::checkPartnerLink()
   {
     if (globals::ASTE_partnerLinkNames.find(toString(*scope) + "." + partnerLinkName) != globals::ASTE_partnerLinkNames.end())
     {
-      string partnerLinkType = globals::ASTEmap[ globals::ASTE_partnerLinks[partnerLinkName] ]->attributes["partnerLinkType"];
-      string myRole          = globals::ASTEmap[ globals::ASTE_partnerLinks[partnerLinkName] ]->attributes["myRole"];
-      string partnerRole     = globals::ASTEmap[ globals::ASTE_partnerLinks[partnerLinkName] ]->attributes["partnerRole"];
+      string partnerLinkType_name = globals::ASTEmap[ globals::ASTE_partnerLinks[partnerLinkName] ]->attributes["partnerLinkType"];
+      string myRole_name          = globals::ASTEmap[ globals::ASTE_partnerLinks[partnerLinkName] ]->attributes["myRole"];
+      string partnerRole_name     = globals::ASTEmap[ globals::ASTE_partnerLinks[partnerLinkName] ]->attributes["partnerRole"];
 
-      if (partnerLinkType != "" && myRole != "" && partnerRole != "")
+      if (partnerLinkType_name != "" && myRole_name != "" && partnerRole_name != "")
       {
-        plRoleDetails = new pPartnerLink(partnerLinkName, partnerLinkType, myRole, partnerRole);
+        plRoleDetails = new pPartnerLink(partnerLinkName, partnerLinkType_name, myRole_name, partnerRole_name);
       }
+
+      // check if operation was defined in WSDL file
+      if (globals::wsdl_filename != "")
+	if (globals::ASTEmap[ globals::ASTE_partnerLinks[partnerLinkName] ]->partnerLinkType != NULL && attributes["operation"] != "")
+	  if (globals::WSDLInfo.checkOperation(globals::ASTEmap[ globals::ASTE_partnerLinks[partnerLinkName] ]->partnerLinkType, attributes["operation"]) == false)
+	    genericError(128, "operation `" + attributes["operation"] + "' referenced in <" + activityTypeName() + "> not defined in WSDL file", attributes["referenceLine"], ERRORLEVER_WARNING);
+
       return (toString(*scope) + "." + partnerLinkName);
     }
   }
@@ -1196,6 +1416,7 @@ string ASTE::activityTypeName() const
     case(K_ONALARM):		return "onAlarm";
     case(K_ONEVENT):		return "onEvent";
     case(K_ONMESSAGE):		return "onMessage";
+    case(K_OPAQUEACTIVITY):	return "opaqueActivity";
     case(K_OTHERWISE):		return "otherwise";
     case(K_PARTNER):		return "partner";
     case(K_PARTNERLINK):	return "partnerLink";
@@ -1239,6 +1460,33 @@ string ASTE::activityTypeName() const
 
 
 
+/*!
+ * \brief returns true if the activity is structured
+ *
+ * \returns boolean
+ */
+bool ASTE::structured()
+{
+  switch (type)
+  {
+    case(K_FLOW):		return true;
+    case(K_FOREACH):		return true;
+    case(K_IF):			return true;
+    case(K_PICK):		return true;
+    case(K_PROCESS):		return true;
+    case(K_REPEATUNTIL):	return true;
+    case(K_SCOPE):		return true;
+    case(K_SEQUENCE):		return true;
+    case(K_WHILE):		return true;
+
+    default:			return false; /* should not happen */
+  }
+}
+
+
+
+
+
 void ASTE::output()
 {
   if (drawn)
