@@ -31,14 +31,13 @@
  *
  * \since   2005/10/18
  *
- * \date    \$Date: 2007/04/20 09:19:20 $
+ * \date    \$Date: 2007/05/11 10:36:33 $
  *
  * \note    This file is part of the tool BPEL2oWFN and was created during the
  *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
- *          http://www.informatik.hu-berlin.de/top/forschung/projekte/tools4bpel
- *          for details.
+ *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.160 $
+ * \version \$Revision: 1.175 $
  */
 
 
@@ -81,11 +80,14 @@ using std::map;
  *****************************************************************************/
 
 extern int frontend_parse();			// from Bison
+extern int frontend_chor_parse();		// from Bison
+extern int frontend_wsdl_parse();		// from Bison
 extern int frontend_debug;			// from Bison
 extern int frontend_nerrs;			// from Bison
 extern int frontend_lineno;			// from Bison
 extern int frontend__flex_debug;		// from flex
 extern FILE *frontend_in;			// from flex
+extern void frontend_restart(FILE*);		// from flex
 
 
 
@@ -95,8 +97,9 @@ extern FILE *frontend_in;			// from flex
  * Global variables
  *****************************************************************************/
 
-/// The Petri Net
+/// The Petri Nets
 PetriNet PN = PetriNet();
+PetriNet PN2 = PetriNet();
 
 /******************************************************************************
  * program parts
@@ -129,15 +132,15 @@ void analyze_cl(int argc, char *argv[])
 
 
 // opening a file
-void open_file(set< string >::iterator file) 
+void open_file(string file) 
 {
   if (inputfiles.size() >= 1)
   {
-    globals::filename = *file;
+    globals::filename = file;
     if (!(frontend_in = fopen(globals::filename.c_str(), "r"))) 
     {
       cerr << "Could not open file for reading: " << globals::filename.c_str() << endl;
-      exit(1);
+      exit(2);
     }
   }
 }
@@ -146,7 +149,7 @@ void open_file(set< string >::iterator file)
 
 
 // closing a file
-void close_file(set< string >::iterator file) 
+void close_file(string file) 
 {
   if ( globals::filename != "<STDIN>" && frontend_in != NULL)
   {
@@ -188,7 +191,7 @@ void finish_AST()
 
 
 // output of every single processed file
-void single_output(set< string >::iterator file, PetriNet PN2) 
+void single_output(set< string >::iterator file)
 {
   // print the AST?
   if (modus == M_AST)
@@ -211,6 +214,34 @@ void single_output(set< string >::iterator file, PetriNet PN2)
     globals::AST->print();
   }
 
+  // print the Visualization?
+  if (modus == M_VIS)
+  {
+    trace(TRACE_INFORMATION, "-> Printing the process visualization ...\n");
+    if (formats[F_DOT])
+    {
+      if (globals::output_filename != "")
+      {
+        output = openOutput(globals::output_filename + "." + suffixes[F_DOT]);
+      }
+      trace(TRACE_INFORMATION, "-> Printing it in dot format ...\n");
+      globals::AST->unparse(kc::printer, kc::visualization);
+      if (globals::output_filename != "")
+      {
+        closeOutput(output);
+        output = NULL;
+      }
+      #ifdef HAVE_DOT
+      string systemcall = "dot -q -Tpng -o" + globals::output_filename + ".png " + globals::output_filename + "." + suffixes[F_DOT];
+      trace(TRACE_INFORMATION, "Invoking dot with the following options:\n");
+      trace(TRACE_INFORMATION, systemcall + "\n\n");
+      system(systemcall.c_str());
+      #endif
+    }
+    else
+    globals::AST->print();
+  }
+
   if (modus == M_PRETTY)
   {
     if (formats[F_XML])
@@ -231,20 +262,73 @@ void single_output(set< string >::iterator file, PetriNet PN2)
 
   // generate and process the control flow graph?
   if (modus == M_CFG)
-  processCFG();
+    processCFG();
 
   // print information about the process
   show_process_information();
 
-  // generate a Petri net?
-  if (modus == M_PETRINET || modus == M_CONSISTENCY)
+
+
+
+  if (modus == M_PETRINET || modus == M_CONSISTENCY) //)
+  {
+    // choose Petri net patterns
+    if (globals::parameters[P_COMMUNICATIONONLY] == true)
+      globals::AST->unparse(kc::pseudoPrinter, kc::petrinetsmall);
+    else
+      globals::AST->unparse(kc::pseudoPrinter, kc::petrinetnew);
+
+    // calculate maximum occurences
+    PN.calculate_max_occurrences();
+
+    // structurally reducte the Petri net
+    if (globals::reduction_level > 0)
+    {
+      trace(TRACE_INFORMATION, "-> Structurally simplifying Petri Net ...\n");
+      PN.reduce(globals::reduction_level);
+    }
+
+    if (modus == M_CONSISTENCY)
+    {
+      // case 1: only one instance of this process is needed
+      if (globals::instances_of_current_process == 1)
+      {
+	// add a prefix and compose PN to PN2
+	PN.addPrefix(globals::ASTEmap[1]->attributes["name"] + ".");
+	PN2.compose(PN);
+      }
+      else // case 2: two or more instances of this process are needed
+      {
+	for(unsigned int instance = 1; instance <= globals::instances_of_current_process; instance++)
+	{
+	  std::cerr << "instance " << instance << "/" << globals::instances_of_current_process << " " << PN2.information() << std::endl;
+	
+	  PetriNet PN3 = PN;
+	  PN3.addPrefix(globals::ASTEmap[1]->attributes["name"] + "_" + toString(instance) + ".");
+	  PN3.add_interface_suffix(".instance_" + toString(instance));
+	  PN2.compose(PN3);
+	}      
+      }
+    
+      // reset data structures
+      PN = PetriNet();
+      globals::AST = NULL;
+    }
+  }
+
+
+/*
+
+  // generate a Petri net (w/o BPEL4Chor)?
+  if ((globals::choreography_filename == "") &&
+    (modus == M_PETRINET || modus == M_CONSISTENCY))
   {
     trace(TRACE_INFORMATION, "-> Unparsing AST to Petri net ...\n");
     // choose Petri net patterns
     if (globals::parameters[P_COMMUNICATIONONLY] == true)
-    globals::AST->unparse(kc::pseudoPrinter, kc::petrinetsmall);
+      globals::AST->unparse(kc::pseudoPrinter, kc::petrinetsmall);
     else
-    globals::AST->unparse(kc::pseudoPrinter, kc::petrinetnew);
+      globals::AST->unparse(kc::pseudoPrinter, kc::petrinetnew);
 
     // calculate maximum occurences
     PN.calculate_max_occurrences();
@@ -257,40 +341,39 @@ void single_output(set< string >::iterator file, PetriNet PN2)
       {
         prefix = file->substr(pos2 + 1, pos - pos2 - 1) + "_";
       }
-      // apply structural reduction rules?
-      if (globals::parameters[P_REDUCE])
+
+      if (globals::reduction_level > 0)
       {
-        trace(TRACE_INFORMATION, "-> Structurally simplifying Petri Net ...\n");
-        PN.reduce();
+	trace(TRACE_INFORMATION, "-> Structurally simplifying Petri Net ...\n");
+	PN.reduce(globals::reduction_level);
       }
+
       PN.addPrefix(prefix);
       PN2.compose(PN);
       PN = PetriNet();
 
       globals::AST = NULL;
       }
-    }
+    }*/
 }
 
 
 
 
 // Final Output of the result
-void final_output( PetriNet PN2) 
+void final_output()
 {
   if (modus == M_CONSISTENCY)
     PN = PN2;
 
   if (modus == M_PETRINET || modus == M_CONSISTENCY)
   {
-    // apply structural reduction rules?
-    if ( globals::parameters[P_REDUCE] )
+    if (globals::reduction_level > 0)
     {
       trace(TRACE_INFORMATION, "-> Structurally simplifying Petri Net ...\n");
-      PN.reduce();
+      PN.reduce(globals::reduction_level);
     }
 
-
     // now the net will not change any more, thus the nodes are re-enumerated
     // and the maximal occurrences of the nodes are calculated.
     PN.reenumerate();
@@ -462,7 +545,9 @@ void final_output( PetriNet PN2)
 	output = openOutput(globals::output_filename + "." + suffixes[F_DOT]);
       }
       trace(TRACE_INFORMATION, "-> Printing Petri net for dot ...\n");
-      PN.set_format(FORMAT_DOT);
+      // if parameter "nointerface" is set, set dot format with parameter "false"
+      // not to draw an interface
+      PN.set_format(FORMAT_DOT, !globals::parameters[P_NOINTERFACE]);
       (*output) << PN;
       if (globals::output_filename != "")
       {
@@ -499,6 +584,9 @@ void final_output( PetriNet PN2)
 }
 
 
+
+
+
 /******************************************************************************
  * main() function
  *****************************************************************************/
@@ -514,7 +602,9 @@ void final_output( PetriNet PN2)
  * \returns 0 if everything went well
  * \returns 1 if an error occurred
  *
- * \todo This function is definitly too long. It should be partitioned!
+ * \todo The opening and closing of files is awkward.
+ *
+ * \bug  Piping does not always work...
  */
 int main( int argc, char *argv[])
 {
@@ -524,26 +614,83 @@ int main( int argc, char *argv[])
   // analyzation of the commandline
   analyze_cl(argc,argv);
 
+
+
+  /*
+   * Choreography: parse a BPEL4Chor file
+   */
+  if (globals::choreography_filename != "")
+  {
+    open_file(globals::choreography_filename);
+
+    show_process_information_header();
+
+    // invoke Bison BPEL4Chor parser
+    trace(TRACE_INFORMATION, "Parsing " + globals::choreography_filename + " ...\n");
+    int parse_result = frontend_chor_parse();
+    trace(TRACE_INFORMATION, "Parsing of " + globals::choreography_filename + " complete.\n");
+
+    close_file(globals::choreography_filename);
+
+    if (parse_result != 0)
+      genericError(122, "", toString(frontend_lineno), ERRORLEVER_WARNING);
+  }
+
+
+
+  /*
+   * parse an optional WSDL file
+   */
+  if (globals::wsdl_filename != "")
+  {
+    open_file(globals::wsdl_filename);
+
+    show_process_information_header();
+
+    // invoke Bison BPEL4Chor parser
+    trace(TRACE_INFORMATION, "Parsing " + globals::choreography_filename + " ...\n");
+    int parse_result = frontend_wsdl_parse();
+    trace(TRACE_INFORMATION, "Parsing of " + globals::choreography_filename + " complete.\n");
+
+    close_file(globals::wsdl_filename);
+
+    if (parse_result != 0)
+      genericError(123, "", toString(frontend_lineno), ERRORLEVER_WARNING);
+
+    show_wsdl_information();
+  }
+  
+
+
   // parsing all inputfiles
   set< string >::iterator file = inputfiles.begin();
   do
   {
-    open_file(file);
+    if (file != inputfiles.end())
+    {
+      open_file(*file);
+
+      // reset the parser
+      frontend_restart(frontend_in);      
+    }
 
     show_process_information_header();
 
-    // invoke Bison parser
+
+    // invoke BPEL Bison parser
     trace(TRACE_INFORMATION, "Parsing " + globals::filename + " ...\n");
     frontend_parse();
     trace(TRACE_INFORMATION, "Parsing of " + globals::filename + " complete.\n");
 
     globals::parsing = false;
-    close_file(file);
+
+    if (file != inputfiles.end())
+      close_file(*file);
 
     if (frontend_nerrs == 0)
     {
       finish_AST();
-      single_output(file, PN2);
+      single_output(file);
     }
     else /* parse error */
     {
@@ -552,14 +699,14 @@ int main( int argc, char *argv[])
 	genericError(104, "", toString(frontend_lineno), ERRORLEVEL_CRITICAL);
 
 	cleanup();
-    	return 1;
+    	return 3;
       }
       else
       {
   	genericError(105, "", toString(frontend_lineno), ERRORLEVEL_NOTICE);
 
 	finish_AST();
-	single_output(file, PN2);
+	single_output(file);
       }
     }
 
@@ -569,7 +716,10 @@ int main( int argc, char *argv[])
 
   trace(TRACE_INFORMATION, "All files have been parsed.\n");
 
-  final_output(PN2);
+  final_output();
+
+  if (debug_level != TRACE_ERROR)
+    cerr << "==============================================================================" << endl << endl;
 
   // everything went fine
   return 0;
