@@ -30,13 +30,13 @@
  *
  * \since   2005/11/09
  *          
- * \date    \$Date: 2007/04/20 14:31:55 $
+ * \date    \$Date: 2007/05/11 14:59:47 $
  * 
  * \note    This file is part of the tool BPEL2oWFN and was created during the
  *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.74 $
+ * \version \$Revision: 1.99 $
  *
  * \ingroup debug
  */
@@ -157,7 +157,9 @@ void show_process_information()
   if (debug_level == TRACE_ERROR)
     return;
 
-  cerr << "------------------------------------------------------------------------------" << endl;
+  if (frontend_nerrs + globals::static_analysis_errors + globals::other_errors > 0)
+    cerr << "------------------------------------------------------------------------------" << endl;
+
   cerr << globals::process_information.basic_activities +
     globals::process_information.structured_activities +
     globals::process_information.scopes << " activities";
@@ -199,7 +201,7 @@ void show_process_information()
   if (globals::static_analysis_errors == 0)
   {
     cerr << colorconsole::fg_green << "[STATIC ANALYSIS] " << colorconsole::fg_standard;
-    cerr << "No errors found checking 44 statics analysis requirements." << endl;
+    cerr << "No errors found checking 51 statics analysis requirements." << endl;
   }
   else if (globals::abstract_process)
   {
@@ -223,7 +225,30 @@ void show_process_information()
     cerr << "Found " << globals::other_errors << " further errors." << endl;
   }
 
-  cerr << "------------------------------------------------------------------------------" << endl << endl;
+  if (modus == M_PETRINET || modus == M_CONSISTENCY)
+    cerr << "------------------------------------------------------------------------------" << endl;
+}
+
+
+
+
+
+/*!
+ * \brief print information about the proces
+ */
+void show_wsdl_information()
+{
+  if (debug_level == TRACE_ERROR)
+    return;
+
+  cerr << globals::wsdl_information.imports << " imports, ";
+  cerr << globals::wsdl_information.types << " types, ";
+  cerr << globals::wsdl_information.bindings << " bindings, ";
+  cerr << globals::wsdl_information.properties << " properties" << endl;
+  cerr << globals::wsdl_information.messages << " messages, ";
+  cerr << globals::wsdl_information.portTypes << " portTypes, ";
+  cerr << globals::wsdl_information.operations << " operations, ";
+  cerr << globals::wsdl_information.partnerLinkTypes << " partnerLinkTypes" << endl;  
 }
 
 
@@ -399,7 +424,94 @@ void genericError(unsigned int code, string information, string line, error_leve
 	  break; }
 
     case(116): // <opaqueActivity> replaced by <empty>
-	{ cerr << "replaced <opaqueActivity> with <empty> activity" << endl;
+	{ cerr << "assuming <empty> semantics for <opaqueActivity>" << endl;
+	  break; }
+
+    case(117): // ignoring attribute suppressJoinFailure="no"
+	{ cerr << "ignoring attribute `suppressJoinFailure' set to \"no\" in <" << information << ">; modeling \"yes\"" << endl;
+	  break; }
+
+    case(118): // ignoring user-defined transition condition (set to true instead)
+	{ cerr << "ignoring user-defined <transitionCondition> in <" << information << ">; modeling \"true\"" << endl;
+	  break; }
+
+    case(119): // ignoring user-defined transition condition (set to XOR instead)
+	{ cerr << "ignoring user-defined <transitionCondition> in <" << information << ">; modeling \"1-out-of-n\"" << endl;
+	  break; }
+
+    case(120): // ignoring <scope> handlers in mode communicationonly
+	{ cerr << "FTC handlers of <scope> activity are not modeled" << endl;
+	  break; }
+
+    case(121): // ignoring activity in mode communicationonly
+	{ cerr << "<" << information << "> activity is modeled by <empty> activity" << endl;
+	  break; }
+
+    case(122): // syntax error in chorography file
+	{ cerr << "syntax error in chorography file" << endl;
+	  break; }
+
+    case(123): // syntax error in WSDL file
+	{ cerr << "syntax error in WSDL file" << endl;
+	  break; }
+
+    case(124): // ignoring XML Schema
+	{ if (information == "/")
+	    cerr << "skipped empty XML Schema element" << endl;
+	  else
+	    cerr << "skipped XML Schema element: <" << information << ">" << endl;
+	  break; }
+
+    case(125): // ignoring variable property element
+	{ cerr << "skipped variable property element: <" << information << ">" << endl;
+	  break; }
+
+    case(126): // WSDL message not found
+	{ cerr << "<message> `" << information << "' referenced in <operation> not defined before" << endl;
+	  break; }
+
+    case(127): // WSDL portType not found
+	{ cerr << "<portType> `" << information << "' referenced in <role> not defined before" << endl;
+	  break; }
+
+    case(128): // WSDL operation not found
+	{ cerr << information << endl;
+	  break; }
+
+    case(129): // WSDL partnerLinkType was found, but role was not found
+	{ cerr << information << endl;
+	  break; }	  
+
+    case(130): // WSDL partnerLinkType was not found
+	{ cerr << information << endl;
+	  break; }	  
+
+    case(131): // BPEL4Chor activity name is not part of a messageLink
+	{ cerr << information << endl;
+	  break; }
+
+    case(132): // activity has neither a name nor a BPEL4Chor id
+	{ cerr << "activity <" + information + "> has neither a `name' nor an `id' attribute to reference a BPEL4Chor <messageLink>" << endl;
+	  break; }
+
+    case(133): // <extensionActivity> replaced by <opaqueActivity>
+	{ cerr << "replacing <extensionActivity> with <opaqueActivity>" << endl;
+	  break; }
+
+    case(134): // <participantType> already definied
+	{ cerr << "BPEL4Chor <participantType> `" << information << "' already defined" << endl;
+	  break; }
+
+    case(135): // BPEL4Chor <participantType> not found for a <participant>
+	{ cerr << information << endl;
+	  break; }
+
+    case(136): // BPEL4Chor forEach id not found
+	{ cerr << "<forEach> id or name `" << information << "' does not reference a BPEL4Chor <participant> or <participantSet>" << endl;
+	  break; }
+
+    case(137): // BPEL4Chor XMLNS not found (either in BPEL4Chor file or in process)
+	{ cerr << information << endl;
 	  break; }
   }
 
@@ -461,6 +573,18 @@ void SAerror(unsigned int code, string information, int lineNumber)
 
   switch (code)
   {
+    case(2):
+      { cerr << "operation `" << information << "' is overloaded in the WSDL file" << endl;
+	break; }
+ 
+    case(3):
+      { cerr << "<catch> must not be applied to the standard fault `" << information << "' when parent scope has attribute `exitOnStandardFault' set to `yes'" << endl;
+	break; }
+	
+    case(5):
+      { cerr << information << " does not match the portType specified by the combination of partnerLink and role in the WSDL description" << endl;
+	break; }
+
     case(6):
       { cerr << "<rethrow> activity must only be used within <catch> or <catchAll>" << endl;
 	break; }
@@ -501,6 +625,22 @@ void SAerror(unsigned int code, string information, int lineNumber)
       { cerr << "<variable> `" << information << "' must either use `messageType', `type' or `element' attribute" << endl;
 	break; }
 	
+    case(32):
+      { cerr << "<" << information << "> has an invalid combination of attributes and/or elements" << endl;
+	break; }
+
+    case(35):
+      { cerr << "attribute `endpointReference' in <from> element is only allowed to be `myRole' if a `myRole' attribute is definied for the used <partnerLink> `" << information << "'" << endl;
+	break; }
+
+    case(36):
+      { cerr << "attribute `endpointReference' in <from> element is only allowed to be `partnerRole' if a `partnerRole' attribute is definied for the used <partnerLink> `"<< information << "'"<< endl;
+	break; }
+
+    case(37):
+      { cerr << "<partnerLink> `" << information << "' referenced in a <to> element must define the attribute `partnerRole'" << endl;
+	break; }
+
     case(44):
       { cerr << "<correlationSet> `" << information << "' already defined in this scope/process" << endl;
 	break; }
@@ -691,7 +831,7 @@ void error()
     trace("Please check logfile for detailed information!\n\n");
   }
 
-  exit(1);
+  exit(4);
 }
 
 
