@@ -25,17 +25,17 @@
  *
  * \author  Niels Lohmann <nlohmann@informatik.hu-berlin.de>,
  *          Christian Gierds <gierds@informatik.hu-berlin.de>,
- *          last changes of: \$Author: znamirow $
+ *          last changes of: \$Author: nielslohmann $
  * 
  * \since   2005/11/11
  *
- * \date    \$Date: 2007/03/29 14:27:40 $
+ * \date    \$Date: 2007/05/09 14:40:33 $
  * 
  * \note    This file is part of the tool BPEL2oWFN and was created during the
  *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.76 $
+ * \version \$Revision: 1.80 $
  *
  * \ingroup debug
  */
@@ -195,3 +195,22 @@ unsigned int max(unsigned int a, unsigned int b)
   else
     return b;
 }
+
+
+
+
+
+/*!
+ * \param s  a string that might be prefixed with an XML namespace
+ * \return   string s without XML namespace prefix
+ *
+ * \note  The prefix "http:" is not removed.
+ */
+string strip_namespace(string s)
+{
+  if (s.find_first_of(":") != string::npos &&
+     (s.substr(0, s.find_first_of(":")) != "http"))
+      return s.substr(s.find_first_of(":")+1, s.length());
+  else
+    return s;
+}
