@@ -29,13 +29,13 @@
  * 
  * \since   2005/11/11
  *
- * \date    \$Date: 2007/03/16 07:17:16 $
+ * \date    \$Date: 2007/05/06 15:48:28 $
  * 
  * \note    This file is part of the tool BPEL2oWFN and was created during the
  *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.48 $
+ * \version \$Revision: 1.51 $
  */
 
 
@@ -90,13 +90,15 @@ int toInt(string s);
 /// converts string to unsigned int
 unsigned int toUInt(string s);
 
+/// strip an XML namespace
+string strip_namespace(string s);
+
 /// returns the maximum of two unsigned ints
 unsigned int max(unsigned int a, unsigned int b);
 
 
 
 
-
 /******************************************************************************
  * Set functions (since we're using templates, we cannot move this to a header)
  *****************************************************************************/
