@@ -29,13 +29,13 @@
  *
  * \since   2005/10/18
  *
- * \date    \$Date: 2007/04/19 06:40:48 $
+ * \date    \$Date: 2007/05/09 16:28:01 $
  *
  * \note    This file is part of the tool BPEL2oWFN and was created during the
  *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.74 $
+ * \version \$Revision: 1.82 $
  */
 
 
@@ -117,11 +117,14 @@ static struct option longopts[] =
   { "format",		required_argument, NULL, 'f' },
   { "parameter",	required_argument, NULL, 'p' },
   { "debug",		required_argument, NULL, 'd' },
+  { "reduce",		required_argument, NULL, 'r' },
+  { "choreography",	required_argument, NULL, 'c' },
+  { "wsdl",		required_argument, NULL, 'w' },
   NULL
 };
 
 /// short options (needed by GNU getopt)
-const char *par_string = "hvm:li:of:p:d:";
+const char *par_string = "hvm:li:of:p:d:r:c:w:";
 
 
 
@@ -145,13 +148,15 @@ void print_help()
   trace("Options:\n");
   trace(" -m, --mode=MODE        use the given mode\n");
   trace(" -p, --parameter=PARAM  modify processing with given parameter\n");
-  trace(" -i, --input=FILE       read input from FILE\n");
+  trace(" -i, --input=FILE       read a BPEL process from FILE\n");
   trace(" -o, --output[=NAME]    write output to file (NAME sets filename)\n");
   trace(" -f, --format=FORMAT    create output of the given format\n");
   trace(" -l, --log=NAME         create log file (NAME sets filename)\n");
   trace(" -d, --debug=NUMBER     set a debug level (NUMBER=0..4 or \"flex\" or \"bison\")\n");
+  trace(" -r, --reduce=NUMBER    apply structural reduction level (NUMBER=0..5)\n");
   trace(" -h, --help             print this help list and exit\n");
   trace(" -v, --version          print program version and exit\n");
+  trace(" -w, --wsdl=FILE        read a WSDL file from FILE\n");
   trace("\n");
   trace("  MODE is one of the following (at most one mode permitted):\n");
   trace("    petrinet            create a Petri net model\n");
@@ -161,12 +166,13 @@ void print_help()
   trace("    consistency         checks whether services communicate deadlock-freely\n");
   trace("\n");
   trace("  PARAMETER is one of the following (multiple parameters permitted):\n");
-  trace("    reduce              structurally simplify generated Petri net\n");
+  trace("    reduce              structurally simplify generated Petri net (-r5)\n");
   trace("    variables           model BPEL variables\n");
   trace("    standardfaults      model BPEL standard faults\n");
   trace("    fhfaults            model faults in fault handlers \n");
   trace("    communicationonly   only model the communicational behavior\n");
   trace("    xor                 use mutually exclusive transition condition\n");
+  trace("    nointerface         do not draw an interface (for dot output\n");
   trace("\n");
   trace("  FORMAT is one of the following (multiple formats permitted):\n");
   trace("    lola, owfn, dot, pep, apnn, ina, spin, info, pnml, txt, info\n");
@@ -254,6 +260,8 @@ void parse_command_line(int argc, char* argv[])
   validFormats[pair<possibleModi,possibleFormats>(M_CONSISTENCY,F_PNML)] = true;
 
   validFormats[pair<possibleModi,possibleFormats>(M_CFG,F_DOT)] = true;
+  
+  validFormats[pair<possibleModi,possibleFormats>(M_VIS,F_DOT)] = true;
 
   // the programme's name on the commandline
   string progname = string(argv[0]);
@@ -297,6 +305,8 @@ void parse_command_line(int argc, char* argv[])
 		    modus = M_CONSISTENCY;
 		  else if (parameter == "cfg")
 		    modus = M_CFG;
+		  else if (parameter == "visualization")
+		    modus = M_VIS;
 		  else  {
 		    trace(TRACE_ALWAYS, "Unknown mode \"" + parameter+ "\".\n");
 		    trace(TRACE_ALWAYS, "Use -h to get a list of valid modes.\n");
@@ -400,7 +410,7 @@ void parse_command_line(int argc, char* argv[])
 		parameter = string(optarg);
 
 		if ( parameter == "reduce")
-		  globals::parameters[P_REDUCE] = true;
+		  globals::reduction_level = 5;
 		else if (parameter == "communicationonly")
 		  globals::parameters[P_COMMUNICATIONONLY] = true;
 		else if (parameter == "standardfaults")
@@ -415,6 +425,8 @@ void parse_command_line(int argc, char* argv[])
 		  globals::parameters[P_LOOPCOUNT] = true;
 		else if (parameter == "loopcontrol")
 		  globals::parameters[P_LOOPCONTROL] = true;
+		else if (parameter == "nointerface")
+		  globals::parameters[P_NOINTERFACE] = true;
 		else {
 		  trace(TRACE_ALWAYS, "Unknown parameter \"" + parameter +"\".\n");
 		  trace(TRACE_ALWAYS, "Use -h to get a list of valid parameters.\n");
@@ -447,10 +459,37 @@ void parse_command_line(int argc, char* argv[])
 		else {
 		  trace(TRACE_ALWAYS, "Unrecognised debug mode: \"" + parameter +"\"!\n");
 		  trace(TRACE_ALWAYS, "Use -h to get a list of valid debug modes.\n");
-    	  cleanup();
+		  cleanup();
 		  exit(1);
-        }
+	        }
+		break;
+	      }
 
+      case 'r':
+		{
+		  globals::reduction_level = toUInt(string(optarg));
+		  if (globals::reduction_level == UINT_MAX)
+		  {
+		    trace(TRACE_ALWAYS, "Unrecognised reduction mode: \"" + string(optarg) +"\"!\n");
+		    trace(TRACE_ALWAYS, "Define a number between 0 and 6.\n");
+		    cleanup();
+		    exit(1);
+		  }
+
+		break;
+	      }
+
+      case 'c':
+	      {
+		options[O_CHOR] = true;
+		globals::choreography_filename = string(optarg);
+		break;
+	      }
+
+      case 'w':
+	      {
+		options[O_WSDL] = true;
+		globals::wsdl_filename = string(optarg);
 		break;
 	      }
 
@@ -597,6 +636,9 @@ void parse_command_line(int argc, char* argv[])
     case M_CFG:
 	    trace(TRACE_INFORMATION, " - generate CFG and do some analysis\n");
 	    break;
+    case M_VIS:
+	    trace(TRACE_INFORMATION, " - generating activity diagram\n");
+	    break;
     case M_PETRINET:
 	    trace(TRACE_INFORMATION, " - generate Petri net\n");
 	    break;
