@@ -26,17 +26,17 @@
  * 
  * \author  Niels Lohmann <nlohmann@informatik.hu-berlin.de>,
  *          Martin Znamirowski <znamirow@informatik.hu-berlin.de>,
- *          last changes of: \$Author: gierds $
+ *          last changes of: \$Author: nielslohmann $
  *
  * \since   created: 2006-03-16
  *
- * \date    \$Date: 2007/03/28 09:28:45 $
+ * \date    \$Date: 2007/05/03 10:28:42 $
  *
  * \note    This file is part of the tool GNU BPEL2oWFN and was created during
  *          the project Tools4BPEL at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.91 $
+ * \version \$Revision: 1.97 $
  *
  * \ingroup petrinet
  */
@@ -176,10 +176,13 @@ string Node::nodeFullName() const
  */
 string Place::nodeShortName() const
 {
+  if (history[0].find("link") != string::npos || history[0].find("!link") != string::npos)
+    return history[0];
+
   if (type == INTERNAL)
     return ("p" + toString(id));
-  else
-    return history[0];
+  
+  return history[0];
 }
 
 
@@ -352,8 +355,8 @@ string Transition::output_dot() const
 
   switch(type)
   {
-    case(IN):		result += "[fillcolor=orange"; break;
-    case(OUT):		result += "[fillcolor=yellow"; break;
+    case(IN):		result += "[fillcolor=orange "; break;
+    case(OUT):		result += "[fillcolor=yellow "; break;
     case(INOUT):	result += "[fillcolor=gold ";
                         result += "label=<";
                         // the table size depends on the node size of .3 (inch);
@@ -364,10 +367,64 @@ string Transition::output_dot() const
                         result += "</TD></TR><TR>";
                         result += "<TD HEIGHT=\"10\" WIDTH=\"21\" FIXEDSIZE=\"TRUE\" BGCOLOR=\"YELLOW\">";
                         result += "</TD></TR></TABLE> >"; break;
-//    case(INOUT):	result += "[fillcolor=gold"; break;
     case(INTERNAL):	result += "["; break;
   }
 
+
+  // add labels for transitions with singleton history
+
+  // internal activities
+  if (history.size() == 1 && history[0].find("internal.empty") != string::npos)
+    result += "label=\"empty\" fillcolor=gray";
+  if (history.size() == 1 && history[0].find("internal.assign") != string::npos)
+    result += "label=\"asgn\" fillcolor=gray";
+  if (history.size() == 1 && history[0].find("internal.opaqueActivity") != string::npos)
+    result += "label=\"opque\" fillcolor=gray";
+
+  // communicating activities
+  if (history.size() == 1 && history[0].find("internal.receive") != string::npos)
+    result += "label=\"recv\"";
+  if (history.size() == 1 && history[0].find("internal.reply") != string::npos)
+    result += "label=\"reply\"";
+  if (history.size() == 1 && history[0].find("internal.invoke") != string::npos)
+    result += "label=\"invk\"";
+  if (history.size() == 1 && history[0].find("internal.onMessage_") != string::npos)
+    result += "label=\"on\\nmsg\"";
+  if (history.size() == 1 && history[0].find(".onEvent.") != string::npos)
+    result += "label=\"on\\nevent\"";
+
+  // structured activities
+  if (history.size() == 1 && history[0].find("internal.case") != string::npos)
+    result += "label=\"case\" fillcolor=azure2";
+  if (history.size() == 1 && history[0].find("internal.onAlarm") != string::npos)
+    result += "label=\"on\\nalarm\" fillcolor=azure2";
+  if (history.size() == 1 && history[0].find("internal.split") != string::npos)
+    result += "label=\"flow\\nsplit\" fillcolor=azure2";
+  if (history.size() == 1 && history[0].find("internal.join") != string::npos)
+    result += "label=\"flow\\njoin\" fillcolor=azure2";
+  if (history.size() == 1 && history[0].find("internal.leave") != string::npos)
+    result += "label=\"leave\\nloop\" fillcolor=azure2";
+  if (history.size() == 1 && history[0].find("internal.loop") != string::npos)
+    result += "label=\"enter\\nloop\" fillcolor=azure2";
+
+  // everything about links
+  if (history.size() == 1 && history[0].find(".setLinks") != string::npos)
+    result += "label=\"tc\" fillcolor=darkseagreen1";
+  if (history.size() == 1 && history[0].find(".evaluate") != string::npos)
+    result += "label=\"jc\\neval\" fillcolor=darkseagreen1";
+  if (history.size() == 1 && history[0].find(".skip") != string::npos)
+    result += "label=\"skip\" fillcolor=darkseagreen1";
+  if (history.size() == 1 && history[0].find(".reset_false") != string::npos)
+    result += "label=\"reset\\nlink\" fillcolor=darkseagreen1";
+  if (history.size() == 1 && history[0].find(".reset_true") != string::npos)
+    result += "label=\"reset\\nlink\" fillcolor=darkseagreen1";
+  
+
+  // stopping
+  if (history.size() == 1 && history[0].find(".stopped.") != string::npos)
+    result += "label=\"stop\" fillcolor=darksalmon";
+  
+
   if (!labels.empty())
   {
     result += " label=\"{";
@@ -407,7 +464,11 @@ string Place::output_dot() const
   result += " p" + toString(id) + "  \t[";//label=\"\"";
 
 #ifdef USING_BPEL2OWFN
-  string label = nodeShortName();
+  string label;
+  if ( wasExternal != "")
+    label = wasExternal;
+  else
+    label = nodeShortName();
 #else
   string label = nodeName();
 #endif
@@ -438,10 +499,13 @@ string Place::output_dot() const
     result += "fillcolor=cyan";
   else if (historyContains("1.internal.clock"))
     result += "fillcolor=seagreen";
+  else if (wasExternal != "")
+    result += "fillcolor=lightgoldenrod1";
     
   result += "]\n";
 
   result += " p" + toString(id) + "_l\t[shape=none];\n";
+
   if (type == OUT)
     result += " p" + toString(id) + " -> p" + toString(id) + "_l [taillabel=\"" + label + "\"]\n";
   else
@@ -471,9 +535,9 @@ void PetriNet::output_dot(ostream *output, bool draw_interface) const
   assert(output != NULL);
 
   (*output) << "digraph N {" << endl;
-  (*output) << " graph [fontname=\"Helvetica\" nodesep=0.3 ranksep=\"0.2 equally\" label=\"";
+  (*output) << " graph [fontname=\"Helvetica\" nodesep=0.3 ranksep=\"0.2 equally\" fontsize=10 label=\"";
 
-  if (globals::parameters[P_REDUCE])
+  if (globals::reduction_level == 5)
     (*output) << "structurally reduced ";
 
   (*output) << "Petri net generated from " << globals::filename << "\"]" << endl;
@@ -516,7 +580,12 @@ void PetriNet::output_dot(ostream *output, bool draw_interface) const
     else
       (*output) << " p" << (*p)->id << " p" << (*p)->id << "_l";
   }
-  (*output) << "\n  label=\"\" style=dashed" << endl;
+
+  if (draw_interface)
+    (*output) << "\n  label=\"\" style=dashed" << endl;
+  else
+    (*output) << "\n  label=\"\" style=invis" << endl;
+
   (*output) << " }" << endl;
 
 
