@@ -27,17 +27,17 @@
  * \author  Niels Lohmann <nlohmann@informatik.hu-berlin.de>,
  *          Christian Gierds <gierds@informatik.hu-berlin.de>,
  *          Martin Znamirowski <znamirow@informatik.hu-berlin.de>,
- *          last changes of: \$Author: znamirow $
+ *          last changes of: \$Author: gierds $
  *
  * \since   2005-10-18
  *
- * \date    \$Date: 2007/04/05 11:51:40 $
+ * \date    \$Date: 2007/05/10 12:23:16 $
  *
  * \note    This file is part of the tool GNU BPEL2oWFN and was created during
  *          the project Tools4BPEL at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.198 $
+ * \version \$Revision: 1.205 $
  *
  * \ingroup petrinet
  */
@@ -565,7 +565,12 @@ PetriNet::~PetriNet()
  */
 Place *PetriNet::newPlace(string my_role, communication_type my_type)
 {
-  Place *p = new Place(getId(), my_role, my_type);
+  string my_role_with_suffix = my_role;
+
+  if (my_role != "" && !forEach_suffix.empty())
+    my_role_with_suffix += ("." + forEach_suffix[0]);
+
+  Place *p = new Place(getId(), my_role_with_suffix, my_type);
   assert(p != NULL);
 
   // Decide in which set of places the place has to be inserted.
@@ -579,8 +584,8 @@ Place *PetriNet::newPlace(string my_role, communication_type my_type)
   // Test if the place is already defined.
   if (my_role != "")
   {
-    assert(roleMap[my_role] == NULL);
-    roleMap[my_role] = p;
+    assert(roleMap[my_role_with_suffix] == NULL);
+    roleMap[my_role_with_suffix] = p;
   }
 
   return p;
@@ -603,7 +608,12 @@ Place *PetriNet::newPlace(string my_role, communication_type my_type)
  */
 Transition *PetriNet::newTransition(string my_role)
 {
-  Transition *t = new Transition(getId(), my_role);
+  string my_role_with_suffix = my_role;
+
+  if (my_role != "" && !forEach_suffix.empty())
+    my_role_with_suffix += ("." + forEach_suffix[0]);
+
+  Transition *t = new Transition(getId(), my_role_with_suffix);
   assert(t != NULL);
 
   T.insert(t);
@@ -611,8 +621,8 @@ Transition *PetriNet::newTransition(string my_role)
   // Test if the transition is already defined.
   if (my_role != "")
   {
-    assert(roleMap[my_role] == NULL);
-    roleMap[my_role] = t;
+    assert(roleMap[my_role_with_suffix] == NULL);
+    roleMap[my_role_with_suffix] = t;
   }
 
   return t;
@@ -1036,7 +1046,7 @@ void PetriNet::mergeParallelTransitions(Transition *t1, Transition *t2)
  */
 void PetriNet::mergePlaces(Place * & p1, Place * & p2)
 {
-  if (p1 == p2)
+  if (p1 == p2 && p1 != NULL)
     return;
 
   if (p1 == NULL || p2 == NULL)
@@ -1055,6 +1065,7 @@ void PetriNet::mergePlaces(Place * & p1, Place * & p2)
 
   p12->tokens = max(p1->tokens, p2->tokens);
   p12->isFinal = (p1->isFinal || p2->isFinal);
+  p12->wasExternal = p1->wasExternal + p2->wasExternal;
 
   for (vector<string>::iterator role = p1->history.begin(); role != p1->history.end(); role++)
   {
@@ -1348,7 +1359,16 @@ Place *PetriNet::findPlace(string role) const
   if ((p != roleMap.end()) && (p->second->nodeType == PLACE))
     return (static_cast<Place *>(p->second));
   else
-    return NULL;
+  {
+    if (!forEach_suffix.empty())
+    {
+      map< std::string, Node* >::const_iterator p_with_suffix = roleMap.find(role + "." + forEach_suffix[0]);
+      if ((p_with_suffix != roleMap.end()) && (p_with_suffix->second->nodeType == PLACE))
+	return (static_cast<Place *>(p_with_suffix->second));
+    }
+  }
+
+  return NULL;
 }
 
 
@@ -1440,6 +1460,25 @@ unsigned int PetriNet::getId()
 
 
 
+/*!
+ * \brief   adds a suffix to the name of all interface places of the net
+ *
+ * \param   suffix to add to the interface places
+ */
+void PetriNet::add_interface_suffix(string suffix)
+{
+  for (set<Place *>::const_iterator place = P_in.begin(); place != P_in.end(); place ++)
+    (*place)->history[0] += suffix;
+
+  for (set<Place *>::const_iterator place = P_out.begin(); place != P_out.end(); place ++)
+    (*place)->history[0] += suffix;
+}
+
+
+
+
+
+
 /*!
  * \brief   adds a prefix to the name of all nodes of the net
  *
@@ -1453,8 +1492,7 @@ void PetriNet::addPrefix(string prefix)
 
     for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
     {
-      if (*place == NULL)
-	roleMap[(prefix + *name)] = *place;
+      roleMap[(prefix + *name)] = *place;
     }
   }
   for (set< Place * >::iterator place = P_in.begin(); place != P_in.end(); place ++)
@@ -1463,8 +1501,7 @@ void PetriNet::addPrefix(string prefix)
 
     for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
     {
-      if (*place == NULL)
-	roleMap[(prefix + *name)] = *place;
+      roleMap[(prefix + *name)] = *place;
     }
   }
   for (set< Place * >::iterator place = P_out.begin(); place != P_out.end(); place ++)
@@ -1473,8 +1510,7 @@ void PetriNet::addPrefix(string prefix)
 
     for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
     {
-      if (*place == NULL)
-	roleMap[(prefix + *name)] = *place;
+      roleMap[(prefix + *name)] = *place;
     }
   }
 
@@ -1518,6 +1554,7 @@ void PetriNet::compose(const PetriNet &net)
     for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
       roleMap[(newPlace->prefix + *name)] = newPlace;
   }
+
   // add all input places
   for (set< Place * >::iterator place = net.P_in.begin(); place != net.P_in.end(); place ++)
   {
@@ -1538,6 +1575,7 @@ void PetriNet::compose(const PetriNet &net)
     for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
       roleMap[(newPlace->prefix + *name)] = newPlace;
   }
+
   // add all output places
   for (set< Place * >::iterator place = net.P_out.begin(); place != net.P_out.end(); place ++)
   {
@@ -1558,6 +1596,7 @@ void PetriNet::compose(const PetriNet &net)
     for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
       roleMap[(newPlace->prefix + *name)] = newPlace;
   }
+
   // add all transitions
   for (set< Transition * >::iterator transition = net.T.begin(); transition != net.T.end(); transition ++)
   {
@@ -1572,6 +1611,7 @@ void PetriNet::compose(const PetriNet &net)
     for(vector< string >::iterator name = (*transition)->history.begin(); name != (*transition)->history.end(); name++)
       roleMap[ newTransition->prefix + *name ] = newTransition;
   }
+
   // create arcs according to the given "net"
   for (set< Arc * >::iterator arc = net.F.begin(); arc != net.F.end(); arc ++)
   {
@@ -1588,9 +1628,12 @@ void PetriNet::compose(const PetriNet &net)
   set< Place * > eraseP_in;
   set< Place * > additionalP_out;
 
+
+
   // merge appropriate input and output places (same name, different prefixes)
   for (set< Place * >::iterator place = P_in.begin(); place != P_in.end(); place ++)
   {
+    /*
     set< Place * >::iterator oPlace = P_out.begin();
     bool finished = false;
     while ( ! finished && oPlace != P_out.end())
@@ -1601,25 +1644,31 @@ void PetriNet::compose(const PetriNet &net)
       else
 	finished = true;
     }
+    */
 
-   if (finished)
+    Place * oPlace = findPlace("out." + (*place)->nodeFullName().erase(0,3));
+
+    // if (finished)
+    if ( oPlace != NULL )
     {
-      if ( (*place)->prefix != (*oPlace)->prefix )
+      if ( (*place)->prefix != (oPlace)->prefix )
       {
         (*place)->type = INTERNAL;
         (*place)->history[0] = (*place)->nodeFullName();
         roleMap[(*place)->nodeFullName()] = (*place);
-        (*oPlace)->type = INTERNAL;
-        (*oPlace)->history[0] = (*oPlace)->nodeFullName();
-        roleMap[(*oPlace)->nodeFullName()] = (*oPlace);
+        (oPlace)->type = INTERNAL;
+        (oPlace)->history[0] = (oPlace)->nodeFullName();
+        roleMap[(oPlace)->nodeFullName()] = (oPlace);
+        (oPlace)->wasExternal = (oPlace)->nodeFullName().substr((oPlace)->nodeFullName().find_first_of(".") + 1);
         P.insert(*place);
-        P.insert(*oPlace);
-        mergePlaces((*place)->nodeFullName(), (*oPlace)->nodeFullName());
+        P.insert(oPlace);
+        mergePlaces((*place)->nodeFullName(), (oPlace)->nodeFullName());
         eraseP_in.insert(*place);
-        P_out.erase(*oPlace);
+        P_out.erase(oPlace);
       }
     }
   }
+
   // erase input places, that were merged
   for (set< Place * >::iterator place = eraseP_in.begin(); place != eraseP_in.end(); place ++)
   {
@@ -1948,3 +1997,28 @@ void PetriNet::produce(const PetriNet &net)
   for (set<string>::iterator t = used_labels.begin(); t != used_labels.end(); t++)
     removeTransition(findTransition(*t));
 }
+
+
+
+
+
+/******************************************************************************
+ * Functions to manage <forEach> suffix
+ *****************************************************************************/
+
+
+unsigned int PetriNet::push_forEach_suffix(string suffix)
+{
+  forEach_suffix.push_front(suffix);
+  return forEach_suffix.size();
+}
+
+
+
+
+
+unsigned int PetriNet::pop_forEach_suffix()
+{
+  forEach_suffix.pop_front();
+  return forEach_suffix.size();
+}
