@@ -31,13 +31,13 @@
  *
  * \since   2005/10/18
  *
- * \date    \$Date: 2007/03/25 15:40:21 $
+ * \date    \$Date: 2007/05/10 12:07:17 $
  *
  * \note    This file is part of the tool GNU BPEL2oWFN and was created during
  *          the project Tools4BPEL at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.141 $
+ * \version \$Revision: 1.147 $
  *
  * \ingroup petrinet
  */
@@ -62,9 +62,11 @@
 #include <vector>
 #include <set>
 #include <map>
+#include <deque>
 
 using std::string;
 using std::vector;
+using std::deque;
 using std::set;
 using std::map;
 using std::ostream;
@@ -309,6 +311,9 @@ class Place: public Node
     /// the maximal occurrences of this (communication) place
     unsigned int max_occurrences;
 
+    /// not empty if place was once an communication place and is now internal because of choreographie
+    string wasExternal;
+
     /// DOT-output of the place (used by PetriNet::dotOut())
     string output_dot() const;
 
@@ -446,7 +451,7 @@ class PetriNet
 
 
     /// applies structral reduction rules
-    unsigned int reduce();
+    unsigned int reduce(unsigned int reduction_level = 5);
 
     /// swaps input and output places
     void mirror();
@@ -454,6 +459,9 @@ class PetriNet
     /// adds a prefix to the name of all nodes of the net
     void addPrefix(string prefix);
 
+    /// adds a suffix to the name of all interface places of the net
+    void add_interface_suffix(string suffix);
+
     /// composes a second Petri net
     void compose(const PetriNet &net);
 
@@ -476,6 +484,13 @@ class PetriNet
     void calculate_max_occurrences();
 
 
+    /// add a suffix for a forEach activity
+    unsigned int push_forEach_suffix(string suffix);
+
+    /// remove the last added suffix
+    unsigned int pop_forEach_suffix();
+
+
     /// calculates the preset of a node
     set<Node*> preset(Node *n) const;
 
@@ -582,6 +597,10 @@ class PetriNet
     unsigned int getId();
 
 
+    /// a role suffix for the forEach activity
+    deque<string> forEach_suffix;
+
+
     /// set of internal places of the Petri net
     set<Place *> P;
 
